#include <iostream>
#include <cstdio>
#include <cstring>
#include <errno.h>
#include <cstdio>
#include <unistd.h>
#include <regex>
#include <signal.h>
#include <sys/wait.h>
#include <string>
#include "socklib.h"

using namespace std;

/*TP Noté de LIFASR5, 2018-2019, Nicolas Louvet et Fabien Rico*/

// ===================================================================

// Affiche l'éventuel message passé en argument, en le faisant
// précéder de "## ".
// sortie : sortie STANDARD
void print_msg(const std::string &str = "") {
  std::cout << "## attention, " + str + ".";
  std::cerr << std::endl << std::flush;
}

// Affiche l'éventuel message d'avertissement passé
// en argument. Le message est précédé de "## ".
// sortie : sortie d'ERREUR STANDARD
void print_warn(const std::string &str = "") {
  if(str == "") std::cerr << "## attention !";
  else std::cerr << "## attention, " + str + ".";
  std::cerr << std::endl << std::flush;
}

// Affiche l'éventuel message d'erreur passé en argument,
// suivi du message donné par la variable errno. Le
// message est précédé de "## ".
// sortie : sortie d'ERREUR STANDARD
void print_error(const std::string &str = "") {
  if(str == "") std::cerr << "## erreur : ";
  else std::cerr << "## erreur, " + str + " : ";
  std::cerr << strerror(errno) << std::endl << std::flush;
}

// Affiche l'éventuel message d'erreur passé en argument,
// avec print_error(), puis termine le processus appelant
// avec pour valeur de retour EXIT_FAILURE.
// sortie : sortie d'ERREUR STANDARD
void exit_error(const std::string &str = "") {
  print_error(str);
  exit(EXIT_FAILURE);
}

// ===================================================================

// Supprime les éventuels caractères de retour à la ligne en fin de
// la string line passée par réference.
void strip_line(string &line) {
  if(!line.empty() && line.back() == '\n') line.pop_back();
  if(!line.empty() && line.back() == '\r') line.pop_back();
}

// ===================================================================

// Fonction pour prendre en compte la terminaison des processus fils
// par le processus père. Sera mise en oeuvre dans le gestionnaire
// pour le signal SIGCHLD.
void handler_sigchld(int sig) {
  // TODO7
}

// ===================================================================

int main(int argc, char *argv[]){
  
  int res = -1; // variable utilisée pour la gestion des erreurs
  
  // Vérification du nombre d'arguments
  // TODO1
  
  // Récupération du port, et affichage.
  // TODO2

  // Mise en place du gestionnaire pour la gestion de
  // SIGCHLD par le processus principal (père).
  struct sigaction sa;
  res = sigaction(SIGCHLD, NULL, &sa);
  if (res == -1) exit_error("sigaction");
  // TODO7

  // Mise en place de la socket serveur s.
  // s est la socket d'écoute.
  // TODO3
  int s = -1;
  
  while(true) { // boucle d'attente des clients
    
    // Attente d'un client.
    // TODO4
    int sd = -1; // sd sera la socket de dialogue avec le client.

    string cname, cport;
    get_peer_info(sd, cname, cport); // récup des infos de connection.
    // cname : nom de la machine depuis laquelle le client s'est connecté
    // cport : port depuis lequel le client s'est connecté
    print_msg("un client s'est connecté, il s'agit de " + cname + " sur le port " + cport);
    
    // Le serveur envoie le message de bienvenue au client
    // TODO5

    // Création d'un processus fils pour gérer le dialogue
    // TODO 6
    pid_t pid = 0;
    { // processus fils

      close(s);
      
      while(true) { // boucle de traitement des commandes

        // Réception de la commande
        // README1
        string com; // variable destinée à recevoir une commande
        res = recv_line(sd, com);
        if(res == 0 || res == -1) break;
        strip_line(com);
        cout << "## commande à traiter : '"<< com <<"'" << endl;
        std::smatch match;

        // Cas où la commande est REPEA STR
        if(std::regex_match(com, match, std::regex("REPEA STR.*"))) {
          // TODO8
          continue;
        }

        // Cas où la commande est PRINT STR
        if(std::regex_match(com, match, std::regex("PRINT STR.*"))) {
          // TODO9
          continue;
        }

        // Cas où la commande est REPEA BIN
        if(std::regex_match(com, match, std::regex("REPEA BIN ([0-9]+).*"))) {
          int t = stoi(match[1]); // nombre d'octets à recevoir
          // TODO10
          continue;
        }

        // Cas où la commande est PRINT BIN
        if(std::regex_match(com, match, std::regex("PRINT BIN ([0-9]+).*"))) {
          int t = stoi(match[1]); // nombre d'octets à recevoir
          // TODO11
          continue;
	}

	cerr << "## commande non trouvée" << endl;
	
      } // boucle de traitement des commandes

      // en sortie de la boucle de traitement :
      // - soit res == 0  si le client a coupé la connexion
      // - soit res == -1 si une erreur s'est produite
      if(res == 0) cout << "## le client s'est déconnecté" << endl;
      else if (res == -1) cerr << "## une erreur s'est produite" << endl;
      else cerr << "## on ne devrait pas arriver ici !!!" << endl;
	  
      // fin du processus fils
      close(sd);
      return EXIT_SUCCESS;
      
    } // processus fils
    
  } // boucle d'attente des clients
  
  return EXIT_SUCCESS;

}
