#! /usr/bin/python3

from threading import Thread
import socket
import time
import random
import sys
import os
import os.path
import subprocess
import shlex
import re
import select
import inspect
import psutil

from fcntl import fcntl, F_GETFL, F_SETFL


def ok(msg, decal=""):
    print (decal + "\033[48;5;100m\033[1m{}\033[0m\t{}".format("OK", msg))

def nok(msg, decal=""):
    print (decal + "\033[48;5;196m\033[1m\33[5m{}\033[0m\t{}".format("Not OK", msg))

def printst(msg, decal = ""):
    print (decal + "\033[33;11m*** " + msg + "\033[00m");



def createServerSocket(port, types = socket.AF_INET6, host="localhost"):
    if (port is None):
        raise RuntimeError("Le port fourni '{}' n'est pas bon".format(port))
    ssock = socket.socket(types, socket.SOCK_STREAM)
    ssock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    ssock.bind((host,port))
    ssock.listen(42)
    return ssock

def acceptNewClient(ssock):
    (sock, (client, port, flowinfo, scopid)) = ssock.accept()
    print ("Un client {} {}".format(client, port))
    return sock

def createClientSocket(host, port):
    csock = socket.create_connection((host, port))

    print ("connecté au serveur {}:{}".format(host, port))

    return csock

    
class ReaderWriter:
    """Une classe pour lire proprement sur une socket. Mais écrire mal :-)"""

    def __init__(self, sock, readsize = 10):
        self.sock = sock
        self.readsize = readsize
        (self.host, self.port) = self.sock.getsockname()
        (self.peerhost, self.peerport) = self.sock.getpeername()
        self.buffer = b""

    def readBuffer(self, size = None, timeout=None):
        if size is None:
            size = self.readsize
        if timeout is not None:
            self.sock.settimeout(timeout)
        try:
            res = self.sock.recv(size)
            if (res == b''):
                print("socket réinitialisée")
                return False
            self.buffer = self.buffer + res
        except ConnectionResetError:
            print("socket en erreur")
            return False
        except socket.timeout:
            print("timeout atteint")
            return False
        return True
        
    def readLine(self, stopchar='\n', timeout=None):
        res = b""
        stopchar = stopchar.encode('utf-8')
        while (stopchar not in self.buffer):
            if not (self.readBuffer(timeout=timeout)):
                res = self.buffer
                self.buffer = b""
                return res
        (res, self.buffer) = self.buffer.split(stopchar, 1)
        
        return res+stopchar

    def readAll(self, size, timeout=None):
        res = b""
        while (len(self.buffer)<size):
            if not (self.readBuffer(timeout=timeout)):
                res = self.buffer
                self.buffer = b""
                return res
        res = self.buffer[0:size]
        self.buffer = self.buffer[size:]
        
        return res

    def readAny(self, timeout=None):
        res = b""
        if not (self.readBuffer(timeout=timeout)):
            res = self.buffer
            self.buffer = b""
            return res
        res = self.buffer

        self.buffer = b""
        
        return res

    def sendAll(self, buff):
        if (isinstance(buff, str)):
            send = buff.encode('utf-8')
        elif (isinstance(buff, bytes)):
            send = buff
            
        self.sock.sendall(send)

    def sendAllInst(self, buff):
        nbcoupes = max(1,round(len(buff)/10))
        if (nbcoupes > 10):
            nbcoupes = 10
#        print("nbcoupe = {} {}".format(nbcoupes, len(buff)))
        list = random.sample(range(0, len(buff)), nbcoupes)
        list.sort()

        if (0 not in list):
            list = [0]+list

        for i in range(0, len(list)-1):
            self.sendAll(buff[list[i]:list[i+1]])
            time.sleep(0.1*random.random())
        self.sendAll(buff[list[-1]:])

    def close(self):
        self.sock.close()
        

def RecupReponse(rep):
    """ Récupère les lignes du résultat commançant par ##"""
    lignes = rep.split(b"\n")
    res = [];
    for li in lignes:
        strli = li.decode("utf-8")
        if re.match("^##.*", strli):
           res.append(strli)
    return res


#class ClientTest(Thread):
#    def __init__(self, commande):
#        Thread.__init__(self)
#        self.commande = commande

def runCommand(commande, timeout = 0.1):
    args = shlex.split(commande)
    proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    callerframerecord = inspect.stack()[1]
    frame = callerframerecord[0]
    info = inspect.getframeinfo(frame)
    print ("(pid={})".format(proc.pid, info.function, info.lineno))

    time.sleep(timeout)
    print ()
    proc.kill()
    ret = proc.wait()
    out = proc.communicate()

    return (out, ret)

def verifyContentOutput(out, regex, std=0, decal=""):
    if (std == 0):
        nout = [0,1]
    elif (std==1):
        nout = [0]
    else :
        nout = [1]

    good = False

    for no in nout:
        o = out[no]
        if o is None:
            break
        # print("o='{}'".format(o))
        lignes = RecupReponse(o)
        print(decal + "Résultats affichés par le programme testé :")
        for li in lignes:
            print (decal + "  '{}'".format(li))
            if re.match(regex, li):
                good = True
                break
            if good:
                break
    if (not good):
        print(decal + "L'expression régulière '{}' n'a pas été trouvée dans l'affichage du programme".format(regex))
        if 0 in nout:
            print(decal + "La sortie standard a été testée elle contenait '{}'".format(out[0]))
        if 1 in nout:
            print(decal + "La sortie d'erreur a été testée elle contenait '{}'".format(out[1]))
        
    return good

def verifyContentOutputBin(out, tab, std=0, decal=""):
    if (std == 0):
        nout = [0,1]
    elif (std==1):
        nout = [0]
    else :
        nout = [1]

    good = False

    for no in nout:
        o = out[no]
        if o is None:
            break
        if o.find(tab):
            print("C'est trouvé !")
            good = True
            break
        if good:
            break
    if (not good):
        print(decal + "Le tableau '{}' n'a pas été trouvé dans l'affichage du programme".format(tab))
        if 0 in nout:
            print(decal + "La sortie standard a été testée elle contenait '{}'".format(out[0]))
        if 1 in nout:
            print(decal + "La sortie d'erreur a été testée elle contenait '{}'".format(out[1]))
        
    return good


def verifyOutput(commande, regex, std=0):
    """
    Vérifie la sortie de la commande

    @param commande : la commande à exécuter
    @param regex : la regex recherchée
    @param std : la sortie à regarder (1 =sortie standard, 2 sortie d'erreur, 0 les deux)
    """
    
    (out, osf) = runCommand(commande)
    return verifyContentOutput(out, regex, std)

def testArgs(nom):
    print("\033[34;11m"
        + "##################################################################################\n"
        + "  1ère série de tests : vérification des arguments                                \n"
        + "##################################################################################\n"
        + "\033[00m"
        );
    nbok = 0
    nbnok = 0

    try:

        regexusage = "^## +.*usage.*{}.*$".format(nom)
        port = random.randrange(2000, 2100)
        regexok = "^## .*port.*{}.*$".format(port)

        commande = "./{}".format(nom);
        printst("test de la commande {} (qui doit faire ## usage... sur la sortie d'erreur)".format(commande))
        if (verifyOutput(commande, regexusage, 2)):
            ok("Le programme lancé avec de mauvais arguments affiche bien {}".format(regexusage))
            nbok = nbok+1
        else:
            nok("Le programme lancé avec de mauvais arguments devrait écrire ## usage {} ...".format(nom))
            nbnok = nbnok+1
    
        commande = "./{} {} plop plop".format(nom, port);
        printst("test de la commande {} (qui doit afficher ## usage... sur la sortie d'erreur)".format(commande))
        if (verifyOutput(commande, regexusage, 2)):
            ok("Le programme lancé avec le bon argument affiche bien {}".format(regexusage))
            nbok = nbok+1
        else:
            nok("Le programme lancé avec de mauvais arguments devrait écrire ## usage {} ...".format(nom))
            nbnok = nbnok+1
    
        commande = "./{} {}".format(nom, port);
        printst("test de la commande ./{} (qui doit afficher ## ...port {}...)".format(commande, port))
        if (verifyOutput(commande, regexok, 0)):
            ok("Le programme lancé avec de mauvais arguments affiche bien {}".format(regexok))
            nbok = nbok+1
        else:
            nok("Le programme lancé avec de mauvais arguments devrait écrire ## ... port ...{}...".format(port))
            nbnok = nbnok+1

    except Exception as err:
        print(err)
        nok("Abandon de la série de tests")
        nbok = 0
        nbnok = 3
    
    print("\033[34;11m");
    print("----------------------------------------------------------------------------------")
    print("Résultats du test : {} bon(s) sur {} passé(s)".format(nbok, nbok+nbnok))
    print("(4 tests à passer dans cette série)                                               ")
    print("----------------------------------------------------------------------------------")
    print("\033[00m")
    return (nbok, nbnok)

def verifyBind(nom, port, pid, decal=""):
    """
    Verifie qu'il y a bien un processus de nom 'nom' qui écoute sur le
    port 'port' et avec le pid 'pid'.
    Si le pid donné est -1, le test du pid n'est pas fait.
    """
    nbok  = 0
    nbnok = 0

    try:
        psu = psutil.Process(pid)
        # if (psu.status() != "zombie"):
        #     ok("Le processus existe et n'est pas à l'état zombi", decal)
        #     nbok = nbok+1
        # else:
        #     nok("Le processus est à l'état zombie")
        #     nbnok = nbnok+1
        cnxs = psu.connections()
        if (len(cnxs)==1):
            ok("Le processus existe et a une connexion", decal)
            nbok = nbok+1
        else:
            nok("Le processus a {} connexions au lieu de 1".format(len(cnxs)), decal)
            nbnok = nbnok+1
        if ((cnxs[0].laddr.ip == "0.0.0.0")
            and (cnxs[0].laddr.port == port)):
            ok("Le port utilisé est bien {}".format(port), decal)
            nbok = nbok+1
        else:
            nok("Le processus utilise l'adresse {}:{} au lieu de 0.0.0.0:{}".format(cnxs[0].laddr.ip, cnxs[0].laddr.port, port), decal)
            nbnok = nbnok+1
        if (cnxs[0].status == "LISTEN"):
            ok("La socket est bien en écoute", decal)
            nbok = nbok+1
        else:
            nok("La socket n'est pas en écoute son status est {} au lieu de LISTEN".format(cnxs[0].status),decal)
            nbnok = nbnok+1
    except :
        nok("Il y a eu un problème lors de la recherche du processus en écoute.", decal)
        nbok = 0
        nbnok = 3
    return (nbok,nbnok)



def testBind(nom):
    """
    test le bind du serveur sur le port demandé
    """
    print("\033[34;11m"
        + "##################################################################################\n"
          + "  2ème série de tests : mise à l'écoute du serveur                              \n"
        + "##################################################################################\n"
        + "\033[00m"
        );

    nbok = 0
    nbnok = 0


    printst("le serveur est-il capable de se mettre à l'écoute sur le port demandé ?");
    port = random.randrange(2000, 2100)
    args = ["./{}".format(nom), "{}".format(port)]
    proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    time.sleep(0.01)

    callerframerecord = inspect.stack()[1]
    frame = callerframerecord[0]
    info = inspect.getframeinfo(frame)
    print ("(pid={})".format(proc.pid, info.function, info.lineno))

    try:
        (inbok, inbnok) = verifyBind(nom, port, proc.pid)
        nbok = nbok+inbok
        nbnok = nbnok+inbnok
    
        commande = "./{} {}".format(nom, port);
        printst("test du serveur si la mise à l'écoute est demandée sur un port impossible (déjà occupé)");
        printst("test avec la commande {} (qui doit afficher ## ...erreur...serveur...<numport>... sur la sortie d'erreur)".format(commande))
        regex = r"## .*erreur.*serveur.*{}.*".format(port)
        if (verifyOutput(commande, regex, 2)):
            ok("Le programme lancé sur un port impossible affiche bien {}".format(regex))
            nbok = nbok+1
        else:
            nok("Le programme lancé sur un port occupé devrait afficher une erreur")
            nbnok = nbnok+1

    except Exception as err:
        print(err)
        nok("Abandon de la série de tests")

    print ("destruction du processus père");
    proc.kill()
    ret = proc.wait()

    print("\033[34;11m");
    print("----------------------------------------------------------------------------------")
    print("Résultats de la série de tests : {} bon(s) sur {} passé(s)".format(nbok, nbok+nbnok))
    print("(4 tests à passer dans cette série)                                               ")
    print("----------------------------------------------------------------------------------")
    print("\033[00m")
    return (nbok, nbnok)



def verifyOneClientConnect(proc, nom, addr, port, decal=""):
    """ Vérifie le protocole lorsqu'un client se connecte sur le port demandé
    et renvoie le readerwriter utile pour la suite de la discussion,
    ainsi que les nombres de tests bon ou pas bon au format (RW, nbok, nbnok)
    """
    nbok = 0
    nbnok = 0

    s = createClientSocket(addr, port);
    rw = ReaderWriter(s)
    portc = rw.port
    addrc = rw.host
    time.sleep(0.05)
    
    flags = fcntl(proc.stdout, F_GETFL)
    fcntl(proc.stdout, F_SETFL, flags | os.O_NONBLOCK)
    flags = fcntl(proc.stderr, F_GETFL)
    fcntl(proc.stderr, F_SETFL, flags | os.O_NONBLOCK)

    std = proc.stdout.read()
    err = proc.stderr.read()

    printst("test du message affiché par le serveur sur sa sortie standard à la connexion d'un client", decal)
    printst("la commande ./{} {} doit ouvrir une connexion sur {}:{} et afficher ## ...client...connecte...({}) ... {}".format(nom, port, addr, port, addrc, port), decal)
    regexcli = "## .*client.*connect.*({}).*{}.*".format(addrc, portc)
    if (verifyContentOutput((std,err), regexcli, 0, decal)):
        ok("Une fois contacté, le programme affiche bien {}".format(regexcli), decal)
        nbok = nbok+1
    else:
        nok("Une fois contacté, le programme n'affiche pas".format(regexcli), decal)
        nbnok = nbnok+1

    chaine = "Bienvenue\r\n" 
    printst("test si le serveur envoie bien la chaîne \"Bienvenue\\r\\n\" au client dès sa connexion", decal)
    line = rw.readLine(timeout=0.5).decode('utf-8')
    if (line == "" or line ==b''):
        nok("Aucune chaine n'est reçue ou la chaine reçue est vide", decal)
        nbnok = nbnok+1
    elif (line != chaine):
        nok("La chaine reçue est \"{}\" au lieu de \"Bienvenue\\r\\n\" (attention aux caractères \\r et \\n !)".format(line.encode("utf-8"), chaine.encode("utf-8")), decal)
        nbnok = nbnok+1
    else:
        ok("La chaine reçu est bien \"Bienvenue\\r\\n\"".format(line), decal)
        nbok=nbok+1

    printst("vérifie que le serveur n'envoie rien de plus après l'envoi de \"Bienvenue\\r\\n\" et avant la réponse du client", decal)
    line = rw.readAny(timeout=0.5).decode('utf-8')
    if (line == "" or line ==b''):
        ok("Aucune chaine n'est reçue ou la chaine reçue est vide", decal)
        nbok = nbok+1
    else:
        nok("Des données ont été reçues '{}' alors qu'il ne devrait rien y avoir".format(line), decal)
        nbnok = nbnok+1

    return (rw, nbok, nbnok) 

def verifyOneClientDisconnect(proc, rw, decal=""):
    nbok = 0
    nbnok = 0
    rw.close()

    time.sleep(0.3)
    
    std = proc.stdout.read()
    err = proc.stderr.read()

    printst("Lorsque le client se déconnecte, le serveur doit afficher ## ...client...déconnecté...", decal)
    regexcli = "## .*client.*déconnecté.*"
    if (verifyContentOutput((std,err), regexcli, 0, decal)):
        ok("Une fois contacté, le programme affiche bien {}".format(regexcli), decal)
        nbok = nbok+1
    else:
        nok("Une fois contacté, le programme n'affiche pas".format(regexcli), decal)
        nbnok = nbnok+1

    return (nbok, nbnok)
    
def testClient(nom):
    print("\033[34;11m"
        + "##################################################################################\n"
        + "   3ème série de tests : vérification des connexions                              \n"
        + "##################################################################################\n"
        + "\033[00m"
        );
    nbok = 0
    nbnok = 0

    port = random.randrange(2000, 2100)

    args = ["./{}".format(nom), "{}".format(port)]
    proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    time.sleep(0.05)

    callerframerecord = inspect.stack()[1]
    frame = callerframerecord[0]
    info = inspect.getframeinfo(frame)
    print ("(pid={})".format(proc.pid, info.function, info.lineno))
    try:

        
        printst("le serveur doit accepter la connexion puis la déconnexion d'un client.")
        try:
            (rw, inbok, inbnok) = verifyOneClientConnect(proc, nom, "localhost", port)
            nbok = nbok + inbok
            nbnok = nbnok + inbnok

            (inbok, inbnok) = verifyOneClientDisconnect(proc, rw)
            nbok = nbok + inbok
            nbnok = nbnok + inbnok
        except Exception as err:
            print(err)
            print("Il est possible que votre programme soit \"planté\"")
            nok("Problème à la connexion ou à la déconnexion")
            nbok = 0
            nbnok = 5
        
        printst("le serveur doit accepter une seconde connexion ; après la déconnexion on connecte \"client 1\".")
        try:
            (rw1, inbok, inbnok) = verifyOneClientConnect(proc, nom, "localhost", port, "\t")
        except Exception as err:
            inbnok = 1
            print(err)
            print("Il est possible que votre programme soit \"planté\"")
        if (inbnok == 0):
            ok("Tout s'est bien passé avec la seconde connexion")
            nbok = nbok+2
        else:
            nok("Il y a eu des problèmes pour la seconde connexion")
            nbnok = nbnok+2
            raise Exception("échec à la seconde connexion")
        
        printst("le serveur doit accepter une troisième connexion en parallèle de la première : avant de déconnecter \"client 1\", on connecte un \"client 2\"")
        try:
            (rw2, inbok, inbnok) = verifyOneClientConnect(proc, nom, "localhost", port, "\t")
        except Exception as err:
            inbnok = 1
            print(err)
            print("Il est possible que votre programme soit \"planté\"")
        if (inbnok == 0):
            ok("Tout s'est bien passé avec la troisième connexion")
            nbok = nbok+2
        else:
            nok("Il y a eu des problèmes pour la troisème connexion")
            nbnok = nbnok+2
            raise Exception("échec à la troisième connexion")
        
        printst("on test si des fils ont bien été créés")
        try:
            psu = psutil.Process(proc.pid)
            fils = psu.children()
            if (len(fils) == 2):
                ok("Le processus a bien 2 fils, un par client")
                nbok = nbok+1
            else:
                print("Créer-vous bien un fils pour chaque nouveau client ?")
                print("Avez-vous pensez à supprimer les zombis ?")
                nok("le processus a {} fils au lieu de 2 attention, le nombre de descendant total est {}".format(len(fils), len(psu.children(True))))
                nbnok = nbnok+1
                for f in fils:
                    print("Le status du fils est {}".format(f.status()))
        except Exception as err:
            print(err)
            nok("Problème avec la vérifications des fils")
            nbnok = nbnok+1
        
        try:
            printst("on teste maintenant si les deux fils gèrent bien la déconnexion pour les deux clients")
            printst("pour client 1");
            (inbok, inbnok) = verifyOneClientDisconnect(proc, rw1, "\t")
            nbok = nbok + inbok
            nbnok = nbnok + inbnok
            printst("pour client 2");
            (inbok, inbnok) = verifyOneClientDisconnect(proc, rw2, "\t")
            nbok = nbok + inbok
            nbnok = nbnok + inbnok
        except Exception as err:
            print(err)
            nok("Problème avec la vérifications des fils")
            nbnok = nbnok+4
        
        try :
            printst("on vérifie que les fils se terminent après la déconnexion")
            psu = psutil.Process(proc.pid)
            fils = psu.children()
            if (len(fils) == 0):
                ok("Le processus n'a plus de fils")
                nbok = nbok+1
            else:
                nok("le processus a {} fils au lieu de 0".format(len(fils), len(psu.children(True))))
                nbnok = nbnok+1
                for f in fils:
                    print("Le status du fils est {}".format(f.status()))
        except Exception as err:
            print(err)
            nok("Problème avec la vérifications des fils")
            nbnok = nbnok+1
    except Exception as err:
        print(err)
        nok("Abandon prématuré de la série de tests")

    print("Envoi du signal kill au processus père");
    proc.kill()
    ret = proc.wait()

    print("\033[34;11m");
    print("----------------------------------------------------------------------------------")
    print("Résultats de la série de tests : {} bon(s) sur {} passé(s)".format(nbok, nbok+nbnok))
    print("(12 tests à passer dans cette série)                                              ")
    print("----------------------------------------------------------------------------------")
    print("\033[00m")
    return (nbok, nbnok)

def verifyProtocolREPEA(proc, rw, str, decal=""):
    nbok = 0
    nbnok = 0
    commande = "REPEA STR\r\n"
    printst("Test de la commande '{}' avec la phrase '{}'".format(commande, str), decal);
    rw.sendAllInst(commande+str+"\r\n")
    rep = rw.readLine(timeout=1)
    std = proc.stdout.read()
    err = proc.stderr.read()
    print(decal + "Le processus a écrit '{}' sur la sortie standard, et '{}' sur la sortie d'erreur".format(std, err))
    
    rep = rep.decode("utf-8")
    if (rep == "{}\r\n".format(str)):
        ok("Bien recu le text de la phrase", decal)
        nbok = nbok+1
    else:
        nok("Recu '{}' au lieu de '{}\r\n'".format(rep, str), decal)
        nbnok = nbnok+1
    
    return (nbok, nbnok)


def verifyProtocolPRINT(proc, rw, str, decal=""):
    nbok = 0
    nbnok = 0
    commande = "PRINT STR\r\n"
    printst("test de la commande '{}' avec la phrase '{}'".format(commande, str), decal);
    rw.sendAllInst(commande+str+"\r\n")
    rep = rw.readLine(timeout=1)

    time.sleep(0.5)
    std = proc.stdout.read()
    err = proc.stderr.read()
    print(decal + "Le processus a écrit '{}' sur la sortie standard et '{}' sur la sortie d'erreur".format(std, err))

    rep = rep.decode("utf-8")
    if (rep == "DONE\r\n"):
        ok("Bien recu l'acquittement", decal)
        nbok = nbok+1
    else:
        nok("Recu '{}' au lieu de 'DONE\r\n'".format(rep, str))
        nbnok = nbnok+1

    if (verifyContentOutput((std, err), ".*## {}.*".format(str), 0, decal)):
        ok("La ligne a bien été affichée", decal)
        nbok = nbok+1
    else:
        nok("La ligne n'a pas bien été affichée", decal)
        nbnok = nbnok+1
    
    return (nbok, nbnok)


def verifyProtocolREPEABIN(proc, rw, str, decal=""):
    nbok = 0
    nbnok = 0
    taille = len(str.encode("utf-8"))
    commande = "REPEA BIN {}\r\n".format(taille)
    printst("Test de la commande '{}' avec la donnée '{}'".format(commande, str), decal);
    rw.sendAllInst(commande+str)
    rep = rw.readAll(taille,timeout=1)
    std = proc.stdout.read()
    err = proc.stderr.read()
    print(decal + "Le processus a écrit '{}' sur la sortie standard, et '{}' sur la sortie d'erreur".format(std, err))
    
    rep = rep.decode("utf-8")
    if (rep == "{}".format(str)):
        ok("Bien recu le binaire en retour", decal)
        nbok = nbok+1
    else:
        nok("Recu '{}' au lieu de '{}'".format(rep.encode("utf-8"), str.encode('utf-8')), decal)
        nbnok = nbnok+1
    
    return (nbok, nbnok)

def verifyProtocolPRINTBIN(proc, rw, str, decal=""):
    nbok = 0
    nbnok = 0
    commande = "PRINT BIN {}\r\n".format(len(str.encode("utf-8")))
    printst("Test de la commande '{}' avec la phrase '{}'".format(commande, str), decal);
    rw.sendAllInst(commande+str)
    rep = rw.readLine(timeout=1)

    time.sleep(0.5)
    std = proc.stdout.read()
    err = proc.stderr.read()
    print(decal + "Le processus a écrit '{}' sur la sortie standard et '{}' sur la sortie d'erreur".format(std, err))

    
    rep = rep.decode("utf-8")
    if (rep == "DONE\r\n"):
        ok("Bien recu l'acquittement", decal)
        nbok = nbok+1
    else:
        nok("Recu '{}' au lieu de 'DONE\r\n'".format(rep, str), decal)
        nbnok = nbnok+1

    if (verifyContentOutputBin((std, err), str.encode("utf-8"), 0, decal)):
        ok("La ligne a bien été affichée", decal)
        nbok = nbok+1
    else:
        nok("La ligne n'a pas bien été affichée")
        nbnok = nbnok+1
    
    return (nbok, nbnok)

def testProtocol(nom):
    print("\033[34;11m"
        + "##################################################################################\n"
        + "  4ème série de tests : vérification du protocole                                 \n"
        + "##################################################################################\n"
        + "\033[00m"
        );
    nbok = 0
    nbnok = 0
    
    try:
    
        port = random.randrange(2000, 2100)
    
        args = ["./{}".format(nom), "{}".format(port)]
        proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        time.sleep(0.05)

        callerframerecord = inspect.stack()[1]
        frame = callerframerecord[0]
        info = inspect.getframeinfo(frame)
        print ("#####pid={}#######".format(proc.pid, info.function, info.lineno))

        s = createClientSocket("localhost", port);
        rw = ReaderWriter(s)
        portc = rw.port
        addrc = rw.host
        time.sleep(0.05)
        flags = fcntl(proc.stdout, F_GETFL)
        fcntl(proc.stdout, F_SETFL, flags | os.O_NONBLOCK)
        flags = fcntl(proc.stderr, F_GETFL)
        fcntl(proc.stderr, F_SETFL, flags | os.O_NONBLOCK)

        bv = rw.readLine(timeout = 1)
        std = proc.stdout.read()
        err = proc.stderr.read()
        print("Le processus a écrit '{}' sur la sortie standard et '{}' sur la sortie d'erreur".format(std, err))

        phrases = ["Hop",
                   "Par les mânes de ce bon gros vieux Fénelon, que ce livre est donc mal écrit. C'est fascinant de médiocrité. Ha ha. La plus forte densité de fautes grammaticales au paragraphe carré ! Eblouissant ! Si Gutenberg avait pu avoir la vision de ce machin, il se serait fait plombier-zingueur.",
                   " Achille Talon est un grand homme, quoique ni par le nez ni par la stature. Doté d'une allure de parieur sur hippodrome mais d'un sens aigu de l'humour littéraire ainsi que d'un important cartable d'auto-dérision-par-inadvertance, le voilà qui nous sert régulièrement - pour peu qu'on lise ses albums - ses élucubrations qui feraient rougir le jongleur de lettres de plaisir. De superbes trouvailles linguo-stylisées: Achille Talon n'as pas fini de m'épater.",
                   "Je m'en moque dans des proportions qui vous donneraient une idée de \0l'infini."
        ]

        for ph in phrases:
            (inbok, inbnok) = verifyProtocolREPEA(proc, rw, ph)
            nbok = nbok+inbok
            nbnok = nbnok+inbnok

        for ph in phrases:
            (inbok, inbnok) = verifyProtocolPRINT(proc, rw, ph)
            nbok = nbok+inbok
            nbnok = nbnok+inbnok

        bin = ["\n".join(phrases)]
        bin.append("""
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMWX0Ok0XMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMWdxMMWd'WMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMNxoKoKWc;0l0WMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMxk0dWWNW0Wx:KdWMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMW,xNlxxxxxxxXMkkWWXKXMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMNOkNN0xxk0KKOXWNO;'.lkkcMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMNOOKXdlldxO000K0:.'cKWXcWW:WMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMWlWMW;oOlcc;dK00ll00;cN0,WN0MMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMOccoc0oK00000000000olodNMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMWK'00O000000000000O'WMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMW;x0000000000000000;kMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMo;K00000K0000000000d,WMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMK.O000dkd00000xo0000O.XMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMWK0O0NMW,l00000000000000OkkO0;oMMMN0OOXMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMNddOO0Oxxd.llllood00000000Okkkxdodolx00OdcOMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMN:koo'.lcclk00OkxO00000000000OO0K000ko:.:l0.XMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMlo'xk .lO00000kdk00000000000OxkkO000000O::k.KMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMo:Od,o0K00000kddx00000000000OxxkkO00000K0x.lWMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMWo,,000000000OxkOkO000000000kxxkO00000000Kk.OMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMO.O0000000000O0kOxkO0k00000OOkkO0000000000l.WMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMM;:0K000000000000kkOx0o00OOkOO0000000000000k.0MMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMNk.cK00000000000000O0x0d0k0k0k00000000000000x.WMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMKc0;'00000000000000000O0O0000000000000000000O.:0MMMMMMMMMMMMMMMMMM
MMMMMMMMMMMX,00d.x0000000000000000000000000000000000000x.:0:OMMMMMMMMMMMMMMMMM
MMMMMMMMMMMll000c.d0000000000000000000000000000000000O:.o00k'WMMMMMMMMMMMMMMMM
MMMMMMMMMMM;o00K0o.;O0000000000000000000000000000K0k:.:O000O.WMMMMMMMMMMMMMMMM
MMMMMMMMMMM:;00000O:.;d00K0000000000000000000000kl'.cO000K0d'WMMMMMMMMMMMMMMMM
MMMMMMMMMMMO.O0000K00o,.,oO0000000000000000Oxl;.'cx00000K0O.xMMMMMMMMMMMMMMMMM
MMMMMMMMMMMW:,000K000000kl;;;:clodddolc:;,,,:lx00000000000,cMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMN,:000000000000000OxxddxxkkO000000000000000Kk':WMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMW:'k00000000000000000000000000000000000000l.kWMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMWO';k00000000000000000000000000000000K0o'oWMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMWk;'lk000000000000000000000000000Od;,dNMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMNx:,;ldO0k00000000000000ccol:;cdXMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMNKxoldcx0000000000d:ookKWMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMWk::loool:,,lNMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMWOxxxkKNMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
"""
        )
        for ph in bin:
            (inbok, inbnok) = verifyProtocolREPEABIN(proc, rw, ph)
            nbok = nbok+inbok
            nbnok = nbnok+inbnok

        for ph in bin:
            (inbok, inbnok) = verifyProtocolPRINTBIN(proc, rw, ph)
            nbok = nbok+inbok
            nbnok = nbnok+inbnok
    
        rw.close()

    except Exception as err:
        print(err)
        nok("Abandon prématuré de la série de tests")

    print ("destruction du processus père");
    proc.kill()
    ret = proc.wait()

    print("\033[34;11m");
    print("----------------------------------------------------------------------------------")
    print("Résultats de la série de tests : {} bon(s) sur {} passé(s)".format(nbok, nbok+nbnok))
    print("(18 à passer dans cette série)           ")
    print("----------------------------------------------------------------------------------")
    print("\033[00m")
    return (nbok, nbnok)
    

    
def testAll(nom):
    nbok = 0
    nbnok = 0
    
    (ok, nok) = testArgs(nom)
    nbok = nbok+ok
    nbnok = nbnok+nok

    (ok, nok) = testBind(nom)
    nbok = nbok+ok
    nbnok = nbnok+nok

    (ok, nok) = testClient(nom)
    nbok = nbok+ok
    nbnok = nbnok+nok

    (ok, nok) = testProtocol(nom)
    nbok = nbok+ok
    nbnok = nbnok+nok

    
    print("\033[34;11m");
    print("-----------------------------------------")
    print("-----------------------------------------")
    print("Résultat final : {} bon(s) sur {} passé(s)".format(nbok, nbok+nbnok))
    print("(37 à passer en tout)                    ")
    print("-----------------------------------------")
    print("-----------------------------------------")
    print("\033[00m")
    return (nbok, nbnok)
    

        
if __name__ == "__main__":
    # Test de la classe
    # ssock = createServerSocket(8080)
    # sock = acceptNewClient(ssock)
    # reader = Reader(sock)

    # line = reader.readAll(10)
    # while (line != b""):
    #     print("Lu '{}'".format(line.decode("utf-8")))
    #     line = reader.readAll(10)
    # sock.close()
    
    
    if len(sys.argv) != 2:
        sys.stderr.write("Usage : python3 {} <nom_executable>\n".format(sys.argv[0]))
        exit(1)
    if (not os.path.isfile(sys.argv[1])):
        sys.stderr.write("Le fichier {} n'existe pas\n".format(sys.argv[1]))
        exit(1)

    nom = sys.argv[1]
    
    
    testAll(nom)
